==> dabstick-radio-0.92.tgz: dabstick-radio-0.92/input/dabstick.cpp <==
#
/*
 *
 *    Copyright (C) 2010, 2011, 2012, 2013
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * 	This particular driver is a very simple wrapper around the
 * 	librtlsdr. The librtlsdr is osmocom software and all rights
 * 	are greatly acknowledged
 */


#define READLEN_DEFAULT (4 * 8192)

#include	<QThread>
#include	"rtl-sdr.h"
#include	"dabstick.h"

//
//	For the callback, we do need some environment which
//	is passed through the ctx parameter
//
//	This is the user-side call back function
//	ctx is the calling task
static
void	RTLSDRCallBack (uint8_t *buf, uint32_t len, void *ctx) {
dabstick	*theStick	= (dabstick *)ctx;
int32_t	tmp;

	if ((len != READLEN_DEFAULT) || (theStick == NULL))
	   return;

	tmp = theStick -> _I_Buffer -> putDataIntoBuffer (buf, len);
	theStick -> sampleCounter += len - tmp;
}
//
//	for handling the events in libusb, we need a controlthread
//	whose sole purpose is to process the rtlsdr_read_async function
//	from the lib.
class	Driver : public QThread {
private:
	dabstick	*theStick;
public:

	Driver (dabstick *d) {
	theStick	= d;
	start ();
	}

	~Driver (void) {
	}

private:
virtual void	run (void) {
	rtlsdr_read_async (theStick -> device,
	                   (rtlsdr_read_async_cb_t)&RTLSDRCallBack,
	                   (void *)theStick,
	                   0,
	                   READLEN_DEFAULT);
	}
};
//
//	Our wrapper is a simple classs
	dabstick::dabstick (int32_t rateIn, bool *success) {
int16_t	deviceCount;
int32_t	r;
int	error;

	*success			= false;	// just the default
	open				= false;
	_I_Buffer			= NULL;
	lastFrequency			= KHz (94700);	// just a dummy
	this	-> rateIn		= rateIn;
	this	-> sampleCounter	= 0;
	this	-> vfoOffset		= 0;

	deviceCount 			= rtlsdr_get_device_count ();
	if (deviceCount == 0) {
	   fprintf (stderr, "No devices found\n");
	   return;
	}
//
//	OK, now open the hardware
	r				= rtlsdr_open (&device, 0);
	if (r < 0) {
	   fprintf (stderr, "Opening dabstick failed\n");
	   *success = false;
	   return;
	}
	open				= true;
	r				= rtlsdr_set_sample_rate (device,
	                                                          rateIn);
	if (r < 0) {
	   fprintf (stderr, "Setting samplerate failed\n");
	   *success = false;
	   return;
	}

	r				= rtlsdr_get_sample_rate (device);
	fprintf (stderr, "samplerate set to %d\n", r);

	rtlsdr_set_tuner_gain_mode (device, 1);
	rtlsdr_set_tuner_gain (device, 190);
	_I_Buffer			= new RingBuffer<uint8_t>(1024 * 2048);
	workerHandle			= NULL;
	*success 			= true;
}

	dabstick::~dabstick	(void) {
	if (open)
	   rtlsdr_close (device);
	if (_I_Buffer != NULL)
	   delete _I_Buffer;
	open = false;
}

void	dabstick::setVFOFrequency	(int32_t f) {
int32_t	actualFrequency	= f + vfoOffset;

	lastFrequency	= f;
	(void)rtlsdr_set_center_freq (device, f + vfoOffset);
}

void	dabstick::getVFOFrequency	(int32_t *f) {
	*f = (int32_t)rtlsdr_get_center_freq (device) - vfoOffset;
}
//
//
bool	dabstick::restartReader	(void) {
int32_t	r;

	if (workerHandle != NULL)
	   return true;

	_I_Buffer	-> FlushRingBuffer ();
	r = rtlsdr_reset_buffer (device);
	if (r < 0)
	   return false;

	rtlsdr_set_center_freq (device, lastFrequency + vfoOffset);
	workerHandle	= new Driver (this);
	return true;
}

void	dabstick::stopReader		(void) {
	if (workerHandle == NULL)
	   return;

	rtlsdr_cancel_async (device);
	if (workerHandle != NULL) {
	   while (!workerHandle -> isFinished ()) 
	      usleep (100);

	   delete	workerHandle;
	}

	workerHandle	= NULL;
}
//
//	Note that this function is neither used for the
//	dabreceiver nor for the fmreceiver
//
int32_t	dabstick::setExternalRate	(int32_t newRate) {
int32_t	r;

	if (newRate < 900000) return rateIn;
	if (newRate > 3200000) return rateIn;

	if (workerHandle == NULL) {
	   r	= rtlsdr_set_sample_rate (device, newRate);
	   if (r < 0) {
	      rtlsdr_set_sample_rate (device, rateIn);
	   }

	   r	= rtlsdr_get_sample_rate (device);
	}
	else {	
//	we stop the transmission first
	   stopReader ();
	   r	= rtlsdr_set_sample_rate (device, newRate);
	   if (r < 0) {
	      rtlsdr_set_sample_rate (device, rateIn);
	   }
	   r	= rtlsdr_get_sample_rate (device);
	fprintf (stderr, "samplerate = %d\n", r);
//	ok all set, continue
	   restartReader ();
	}

	rateIn	= newRate;
	return r;
}

static
int gains[] = {-10, 15, 40, 65, 90, 115, 140, 165, 190,
	       215, 240, 290, 340, 420, 430, 450, 470, 490};
//
int32_t	dabstick::setExternalGain	(int32_t gain) {
static int	oldGain	= 0;

	if (gain == oldGain)
	   return gains [oldGain];
	if ((gain < 0) || (gain > 17))
	   return gains [oldGain];	

	oldGain	= gain;
	rtlsdr_set_tuner_gain (device, gains [gain]);
	return rtlsdr_get_tuner_gain (device);
}
//
//	correction is in Hz
void	dabstick::freqCorrection	(int32_t ppm) {
	rtlsdr_set_freq_correction (device, ppm);
}

//
//	The brave old getSamples. For the dab stick, we get
//	size: still in I/Q pairs
int32_t	dabstick::getSamples (uint8_t *V, int32_t size) { 
int32_t	amount;
//
	amount = _I_Buffer	-> getDataFromBuffer (V, 2 * size);
	return amount / 2;
}

//	and especially for our beloved spectrum viewer we provide
int32_t	dabstick::getSamples 	(uint8_t  *v, int32_t n, int32_t segmentSize) {
int32_t	amount;

	amount = _I_Buffer	-> getDataFromBuffer (v, 2 * n);
	_I_Buffer	-> skipDataInBuffer (2 * (segmentSize - n));

	return amount / 2;
}

int32_t	dabstick::Samples	(void) {
	return _I_Buffer	-> GetRingBufferReadAvailable () / 2;
}
//
uint8_t	dabstick::myIdentity		(void) {
	return STICK;
}

int32_t	dabstick::getSamplesMissed	(void) {
int32_t	tmp;

	tmp	= sampleCounter;
	sampleCounter	= 0;
	return tmp;
}

==> dabstick-radio-0.95.tgz: dabstick-radio-0.95/input/dabstick.cpp <==
#
/*
 *
 *    Copyright (C) 2010, 2011, 2012, 2013
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * 	This particular driver is a very simple wrapper around the
 * 	librtlsdr. The librtlsdr is osmocom software and all rights
 * 	are greatly acknowledged
 */

#include	<QThread>
#include	"rtl-sdr.h"
#include	"dabstick.h"

//
//	For the callback, we do need some environment which
//	is passed through the ctx parameter
//
//	This is the user-side call back function
//	With a rate of 2048000 samples/second, the function
//	is apparently called (2048000 / 32000 = ) about 125 times a second
//	ctx is the calling task
static
void	RTLSDRCallBack (uint8_t *buf, uint32_t len, void *ctx) {
dabstick	*theStick	= (dabstick *)ctx;
int32_t	tmp;

	if (theStick == NULL)
	   return;
	if (theStick -> bufferLength != len)
	   return;

	tmp = theStick -> _I_Buffer -> putDataIntoBuffer (buf, len);
	theStick -> sampleCounter += len - tmp;
}
//
//	for handling the events in libusb, we need a controlthread
//	whose sole purpose is to process the rtlsdr_read_async function
//	from the lib.
class	Driver : public QThread {
private:
	dabstick	*theStick;
public:

	Driver (dabstick *d) {
	theStick	= d;
	start ();
	}

	~Driver (void) {
	}

private:
virtual void	run (void) {
	rtlsdr_read_async (theStick -> device,
	                   (rtlsdr_read_async_cb_t)&RTLSDRCallBack,
	                   (void *)theStick,
	                   0,
	                   theStick -> bufferLength);
	}
};
//
//	Our wrapper is a simple classs
	dabstick::dabstick (int32_t rateIn, int32_t l, int32_t n, bool *success) {
int16_t	deviceCount;
int32_t	r;
int	error;

	this	-> rateIn		= rateIn;
	this	-> bufferLength		= l * 8192;
	*success			= false;	// just the default
	open				= false;
	_I_Buffer			= NULL;
	lastFrequency			= KHz (94700);	// just a dummy
	this	-> sampleCounter	= 0;
	this	-> vfoOffset		= 0;
	deviceCount 			= rtlsdr_get_device_count ();
	if (deviceCount == 0) {
	   fprintf (stderr, "No devices found\n");
	   return;
	}
//
//	OK, now open the hardware
	r				= rtlsdr_open (&device, 0);
	if (r < 0) {
	   fprintf (stderr, "Opening dabstick failed\n");
	   *success = false;
	   return;
	}
	open				= true;
	r				= rtlsdr_set_sample_rate (device,
	                                                          rateIn);
	if (r < 0) {
	   fprintf (stderr, "Setting samplerate failed\n");
	   *success = false;
	   return;
	}

	r				= rtlsdr_get_sample_rate (device);
	fprintf (stderr, "samplerate set to %d\n", r);

	rtlsdr_set_tuner_gain_mode (device, 1);
	rtlsdr_set_tuner_gain (device, 190);
//
//	The buffer may contain app 0.5 seconds of data

	_I_Buffer			= new RingBuffer<uint8_t>(n * 1024);
	workerHandle			= NULL;
	*success 			= true;
}

	dabstick::~dabstick	(void) {
	if (open)
	   rtlsdr_close (device);
	if (_I_Buffer != NULL)
	   delete _I_Buffer;
	open = false;
}

void	dabstick::setVFOFrequency	(int32_t f) {
int32_t	actualFrequency	= f + vfoOffset;

	lastFrequency	= f;
	(void)rtlsdr_set_center_freq (device, f + vfoOffset);
}

void	dabstick::getVFOFrequency	(int32_t *f) {
	*f = (int32_t)rtlsdr_get_center_freq (device) - vfoOffset;
}
//
//
bool	dabstick::restartReader	(void) {
int32_t	r;

	if (workerHandle != NULL)
	   return true;

	_I_Buffer	-> FlushRingBuffer ();
	r = rtlsdr_reset_buffer (device);
	if (r < 0)
	   return false;

	rtlsdr_set_center_freq (device, lastFrequency + vfoOffset);
	workerHandle	= new Driver (this);
fprintf (stderr, "dabstick restarted\n");
	return true;
}

void	dabstick::stopReader		(void) {
	if (workerHandle == NULL)
	   return;

	rtlsdr_cancel_async (device);
	if (workerHandle != NULL) {
	   while (!workerHandle -> isFinished ()) 
	      usleep (100);

	   delete	workerHandle;
	}

	workerHandle	= NULL;
}
//
//	Note that this function is neither used for the
//	dabreceiver nor for the fmreceiver
//
int32_t	dabstick::setExternalRate	(int32_t newRate) {
int32_t	r;

	if (newRate < 900000) return rateIn;
	if (newRate > 3200000) return rateIn;

	if (workerHandle == NULL) {
	   r	= rtlsdr_set_sample_rate (device, newRate);
	   if (r < 0) {
	      rtlsdr_set_sample_rate (device, rateIn);
	   }

	   r	= rtlsdr_get_sample_rate (device);
	}
	else {	
//	we stop the transmission first
	   stopReader ();
	   r	= rtlsdr_set_sample_rate (device, newRate);
	   if (r < 0) {
	      rtlsdr_set_sample_rate (device, rateIn);
	   }
	   r	= rtlsdr_get_sample_rate (device);
	fprintf (stderr, "samplerate = %d\n", r);
//	ok all set, continue
	   restartReader ();
	}

	rateIn	= newRate;
	return r;
}

static
int gains[] = {-10, 15, 40, 65, 90, 115, 140, 165, 190,
	       215, 240, 290, 340, 420, 430, 450, 470, 490};
//
int32_t	dabstick::setExternalGain	(int32_t gain) {
static int	oldGain	= 0;

	if (gain == oldGain)
	   return gains [oldGain];
	if ((gain < 0) || (gain > 17))
	   return gains [oldGain];	

	oldGain	= gain;
	rtlsdr_set_tuner_gain (device, gains [gain]);
	return rtlsdr_get_tuner_gain (device);
}
//
//	correction is in Hz
void	dabstick::freqCorrection	(int32_t ppm) {
	rtlsdr_set_freq_correction (device, ppm);
}

//
//	The brave old getSamples. For the dab stick, we get
//	size: still in I/Q pairs
int32_t	dabstick::getSamples (uint8_t *V, int32_t size) { 
int32_t	amount;
//
	amount = _I_Buffer	-> getDataFromBuffer (V, 2 * size);
	return amount / 2;
}

//	and especially for our beloved spectrum viewer we provide
int32_t	dabstick::getSamples 	(uint8_t  *v, int32_t n, int32_t segmentSize) {
int32_t	amount;

	amount = _I_Buffer	-> getDataFromBuffer (v, 2 * n);
	_I_Buffer	-> skipDataInBuffer (2 * (segmentSize - n));

	return amount / 2;
}

int32_t	dabstick::Samples	(void) {
	return _I_Buffer	-> GetRingBufferReadAvailable () / 2;
}
//
uint8_t	dabstick::myIdentity		(void) {
	return STICK;
}

int32_t	dabstick::getSamplesMissed	(void) {
int32_t	tmp;

	tmp	= sampleCounter;
	sampleCounter	= 0;
	return tmp;
}

void	dabstick::resetBuffer		(void) {
	_I_Buffer	-> FlushRingBuffer ();
}
